package org.greenstone.gatherer.collection;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.collection.Collection;
import org.greenstone.gatherer.util.ArrayTools;
import org.greenstone.gatherer.util.Utility;
/** This class provides the functionality to delete current collections from the GSDLHOME/collect/ directory. The user chooses the collection from a list, where each entry also displays details about itself, confirms the delete of a collection by checking a checkbox then presses the ok button to actually delete the collection.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class DeleteCollectionPrompt
    extends JDialog {
    /** The currently selected collection for deletion. */
    private Collection collection = null;
    /** The model behind the list. */
    private DefaultListModel list_model = null;
    /** A reference to ourself so any inner-classes get dispose of us. */
    private DeleteCollectionPrompt prompt = null;
    /** The close button, which exits the prompt without deleting anything. */
    private JButton close_button = null;
    /** The ok button which causes the selected collection to be deleted. */
    private JButton ok_button = null;
    /** The confirmation check box. */
    private JCheckBox confirmation = null;
    /** The label above details. */
    private JLabel details_label = null;
    /** The label above the list. */
    private JLabel list_label = null;
    /** The list of available collections. */
    private JList list = null;
    /** The text area used to display details about the collection selected. */
    private JTextArea details = null;
    /** A string array used to pass arguments to the phrase retrieval method. */
    private String args[] = null;
    /** The size of the delete prompt screen. */
    public static final Dimension SIZE = new Dimension(500, 500);
    /** Constructor.
     * @see org.greenstone.gatherer.collection.DeleteCollectionPrompt.CloseButtonListener
     * @see org.greenstone.gatherer.collection.DeleteCollectionPrompt.CollectionListListener
     * @see org.greenstone.gatherer.collection.DeleteCollectionPrompt.ConfirmationCheckBoxListener
     * @see org.greenstone.gatherer.collection.DeleteCollectionPrompt.OKButtonListener
     */
    public DeleteCollectionPrompt() {
	super();
	this.close_button = new JButton(get("General.Close", null));
	this.confirmation = new JCheckBox(get("Confirm_Delete", null));
	this.details = new JTextArea();
	this.details_label = new JLabel(get("Collection_Details", null));
	this.list = new JList();
	this.list_label = new JLabel(get("Collection_List", null));
	this.list_model = new DefaultListModel();
	this.ok_button = new JButton(get("General.OK", null));
	this.prompt = this;
	this.setModal(true);
	this.setSize(SIZE);
	this.setTitle(get("Title", null));
	close_button.addActionListener(new CloseButtonListener());
	confirmation.addActionListener(new ConfirmationCheckBoxListener());
	confirmation.setEnabled(false);
	confirmation.setSelected(false);
	details.setFont(Gatherer.config.getFont("general.tooltip_font", false));
	details.setText(get("No_Collection", null));
	list.addListSelectionListener(new CollectionListListener());
	list.clearSelection();
	list.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	list.setModel(list_model);
	ok_button.addActionListener(new OKButtonListener());
	ok_button.setEnabled(false);
	scanForCollections();
    }
    /** Destructor. */
    public void destroy() {
	close_button = null;
	confirmation = null;
	details = null;
	details_label = null;
	list_model.clear();
	list_model = null;
	list = null;
	ok_button = null;
	prompt = null;
    }
    /** This method causes the modal prompt to be displayed. */
    public void display() {
	// Central pane
	JPanel list_pane = new JPanel(new BorderLayout());
	list_pane.add(list_label, BorderLayout.NORTH);
	list_pane.add(new JScrollPane(list), BorderLayout.CENTER);
	list_pane.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 0));

	JPanel details_pane = new JPanel(new BorderLayout());
	details_pane.add(details_label, BorderLayout.NORTH);
	details_pane.add(new JScrollPane(details), BorderLayout.CENTER);
	details_pane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));

	JPanel central_pane = new JPanel(new GridLayout(2, 1));
	central_pane.add(list_pane);
	central_pane.add(details_pane);
	central_pane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

	// Lower pane
	JPanel confirmation_pane = new JPanel(new BorderLayout());
	confirmation_pane.add(confirmation, BorderLayout.CENTER);
	confirmation_pane.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));

	JPanel button_pane = new JPanel(new GridLayout(1, 2));
	button_pane.add(ok_button);
	button_pane.add(close_button);
	button_pane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));

	JPanel lower_pane = new JPanel(new BorderLayout());
	lower_pane.add(confirmation_pane, BorderLayout.NORTH);
	lower_pane.add(button_pane, BorderLayout.SOUTH);
	lower_pane.setBorder(BorderFactory.createEmptyBorder(0, 5, 5, 5));

	// Final.
	JPanel content_pane = (JPanel)this.getContentPane();
	content_pane.setLayout(new BorderLayout());
	content_pane.add(central_pane, BorderLayout.CENTER);
	content_pane.add(lower_pane, BorderLayout.SOUTH);

	// Center and display.
	Dimension screen_size = Gatherer.config.screen_size;
	this.setLocation((screen_size.width - SIZE.width) / 2, (screen_size.height - SIZE.height) / 2);
	this.show();
    }
    /** Shows a delete complete prompt. 
	  * @param success A <strong>boolean</strong> indicating if the collection was successfully deleted.
	  * @see org.greenstone.gatherer.collection.Collection
	  */
    public void resultPrompt(boolean success) {
	args = new String[1];
	args[0] = collection.getName();
	if(success) {
	    JOptionPane.showMessageDialog(prompt,get("Successful_Delete", args),get("Successful_Title", null),JOptionPane.INFORMATION_MESSAGE);
	}
	else {
	    JOptionPane.showMessageDialog(prompt,get("Failed_Delete", args),get("Failed_Title", null),JOptionPane.WARNING_MESSAGE);
	}
    }
    /** Retrieves a phrase from the <strong>Dictionary</strong> in <strong>Gatherer</strong>.
	  * @param key A <strong>String</strong> used to match against a specific phrase in the dictionary.
	  * @see org.greenstone.gatherer.Dictionary
	  * @see org.greenstone.gatherer.Gatherer
	  */
    private String get(String key, String args[]) {
	if(key.indexOf(".") == -1) {
	    key = "DeleteCollectionPrompt." + key;
	}
	return Gatherer.dictionary.get(key, args);
    }
    /** Attempts to load the given collection. Currently uses simple
	  * serialization of the collection class.
	  * @param location The path to the serialized collection.
	  */
    public Collection loadCollection(File file) {
	Collection col = null;
	try {
	    FileInputStream fis = new FileInputStream(file);
	    BufferedInputStream bin = new BufferedInputStream(fis);
	    ObjectInputStream input = new ObjectInputStream(bin);
	    col = (Collection) input.readObject();
	    input.close();
	    input = null;
	    bin.close();
	    bin = null;
	    fis.close();
	    fis = null;
	} 
	catch (Exception error) {
	    error.printStackTrace();
	}
	return col;
    }
    /** Method to scan the collect directory retrieving and reloading each collection it finds, while building the list of known collections.
	  * @see org.greenstone.gatherer.Configuration
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.util.ArrayTools
	  * @see org.greenstone.gatherer.util.Utility
	  */
    private void scanForCollections() {
	// Start at the collect dir.
	String collect_directory_name = Utility.getCollectionDir(Gatherer.config.gsdl_path);
	File collect_directory = new File(collect_directory_name);
	if(collect_directory.exists()) {
				// Now for each child directory see if it contains a .col file and
				// if so try to load it..
	    File collections[] = collect_directory.listFiles();
	    ArrayTools.sort(collections);
	    for(int i = 0; collections != null && i < collections.length; i++) {
		File children[] = collections[i].listFiles();
		for(int j = 0; children != null && j < children.length; j++) {
		    if(children[j].getAbsolutePath().endsWith(".col")) {
			Collection col = loadCollection(children[j]);
			// If we've actually loaded 
			if(col != null) {
			    list_model.addElement(col);
			}
		    }
		}
	    }
	}
	// Otherwise the collect directory doesn't actually exist, so there ain't much we can do.
    }
    /** A button listener implementation, which listens for actions on the close button and disposes of the dialog when detected. */
    private class CloseButtonListener 
	implements ActionListener {
	/** Any implementation of ActionListener must include this method so we can be informed when the button is actioned.
	 * @param event An <strong>ActionEvent</strong> containing all the relevant information garnered from the event itself.
			*/
	public void actionPerformed(ActionEvent event) {
	    prompt.dispose();
	}
    }
    /** This private class listens for selection events in from the list and then displays the appropriate details for that collection.
	  */
    private class CollectionListListener 
	implements ListSelectionListener {
	/** Any implementation of ListSelectionListener must include this method so we can be informed when the list selection changes.
	 * @param event A <strong>ListSelectionEvent</strong> containing all the relevant information garnered from the event itself.
			*/
	public void valueChanged(ListSelectionEvent event) {
	    if(!list.isSelectionEmpty()) {
		confirmation.setEnabled(true);
		collection = (Collection) list.getSelectedValue();
		args = new String[4];
		args[0] = collection.getTitle();
		args[1] = collection.getName();
		args[2] = collection.getEmail();
		args[3] = collection.getDescription();
		details.setText(get("Details", args));
	    }
	    else {
		confirmation.setEnabled(false);
		details.setText(get("No_Collection", null));
	    }
	}
    }
    /** A check box listener so we can tell if the users selected confirm. */
    private class ConfirmationCheckBoxListener
	implements ActionListener {
	/** Any implementation of ActionListener must include this method so we can be informed when the button is actioned.
	 * @param event An <strong>ActionEvent</strong> containing all the relevant information garnered from the event itself.
			*/
	public void actionPerformed(ActionEvent event) {
	    if(confirmation.isSelected()) {
		ok_button.setEnabled(true);
	    }
	    else {
		ok_button.setEnabled(false);
	    }
	}
    }		  
    /** The OK button listener implementation. */
    private class OKButtonListener 
	implements ActionListener {
	/** Any implementation of ActionListener must include this method so we can be informed when the button is actioned.
	 * @param event An <strong>ActionEvent</strong> containing all the relevant information garnered from the event itself.
	 * @see org.greenstone.gatherer.Configuration
	 * @see org.greenstone.gatherer.Gatherer
	 * @see org.greenstone.gatherer.util.Utility
	 */
	public void actionPerformed(ActionEvent event) {
				// Delete the selected collection.
	    File delete_me = new File(Utility.getCollectionDir(Gatherer.config.gsdl_path) + collection.getName() + File.separator);
	    if(Utility.delete(delete_me)) {
		resultPrompt(true);
		list_model.removeElement(collection);
		details.setText(get("No_Collection", null));
		confirmation.setEnabled(false);
		confirmation.setSelected(false);
		ok_button.setEnabled(false);
		collection = null;
	    }
	    else {
		resultPrompt(false);
	    }
	}
    }
}


