package org.greenstone.gatherer.gui;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import java.io.*;
import java.util.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.util.ArrayTools;
import org.greenstone.gatherer.util.Utility;
/** This data-only class contains all of the import and build options which persist between sessions, which currently is all of them. 
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class BuildOptions 
    implements Serializable {
    /** Do we include empty classifications? */
    public boolean allclassifications = false;
    /** Have we set an archive directory? */
    public boolean archivedir = false;
    /** Have we set a build directory? */
    public boolean builddir = false;
    /** Do we want debug information? */
    public boolean debug = false;
    /** Have we set a collect directory? */
    public boolean collectdir = false;
    /** Do we want the build process to create images as it goes along? */
    public boolean create_images = false;
    /** Have we set a groupsize value? */
    public boolean groupsize = false;
    /** Do we want the contents of the archive directory zipped? */
    public boolean gzip = false;
    /** Have we set an import directory? */
    public boolean importdir = false;
    /** Have we selected to build the collection on certain indexes? */
    public boolean index = false;
    /** Do we explicitly want to keep the archives dir? */
    public boolean keepold = false;
    /** Have we set a maxdocs value? */
    public boolean maxdocs = false;
    /** Have we selected a certain mode to build in? */
    public boolean mode = false;
    /** Do we build the indexes, but store no compressed text? */
    public boolean notext = false;
    /** Have we decided on a OID type? */
    public boolean oidtype = false;
    /** Have we set an out file? */
    public boolean out = false;
    /** Do we explicitly want to remove the archives dir? */
    public boolean removeold = false;
    /** Have we choosen a metadata element to sort on? */
    public boolean sortmeta = false;
    /** Have we set a verbosity value? */
    public boolean verbosity = false;
    /** How many documents to a hash group in the archive? */
    public int groupsize_value = 1;
    /** What is the maximum number of documents to process? */
    public int maxdocs_value = 1;
    /** What is the desired level of verbosity? */
    public byte verbosity_value = 0;
    /** The archive directory. */
    public String archivedir_value = "";
    /** The building directory. */
    public String builddir_value = "";
    /** The collect directory. */
    public String collectdir_value = "";
    /** The import directory. */
    public String importdir_value = "";
    /** The build mode. */
    public String mode_value = "";
    /** The OID generation method. */
    public String oidtype_value = "";
    /** The name of the file to write std_err to. */
    public String out_value = "";
    /** The name of the metadata element to sort the collection by. */ 
    public String sortmeta_value = "";
    /** The names of classifiers to be indexes for. */
    public String[] index_value = null;
    /** Element of argument type enumeration. */
    static final public byte BUILD_ARGS = 0;
    /** Element of argument type enumeration. */
    static final public byte IMPORT_ARGS = 1;
    /** Element of option type enumeration. */
    static final public byte ALLCLASSIFICATIONS = 0;
    /** Element of option type enumeration. */
    static final public byte ARCHIVEDIR = 1;
    /** Element of option type enumeration. */
    static final public byte BUILDDIR = 2;
    /** Element of option type enumeration. */
    static final public byte DEBUG = 3;
    /** Element of option type enumeration. */
    static final public byte COLLECTDIR = 4;
    /** Element of option type enumeration. */
    static final public byte CREATEIMAGES = 5;
    /** Element of option type enumeration. */
    static final public byte GROUPSIZE = 6;
    /** Element of option type enumeration. */
    static final public byte GZIP = 7;
    /** Element of option type enumeration. */
    static final public byte IMPORTDIR = 8;
    /** Element of option type enumeration. */
    static final public byte INDEX = 9;
    /** Element of option type enumeration. */
    static final public byte KEEPOLD = 10;
    /** Element of option type enumeration. */
    static final public byte MAXDOCS = 11;
    /** Element of option type enumeration. */
    static final public byte MODE = 12;
    /** Element of option type enumeration. */
    static final public byte NOTEXT = 13;
    /** Element of option type enumeration. */
    static final public byte OIDTYPE = 14;
    /** Element of option type enumeration. */
    static final public byte OUT = 15;
    /** Element of option type enumeration. */
    static final public byte REMOVEOLD = 16;
    /** Element of option type enumeration. */
    static final public byte SORTMETA = 17;
    /** Element of option type enumeration. */
    static final public byte VERBOSITY = 18;
    /** This constructor initializes values by using the current configuration and collect if there is one.
     */
    public BuildOptions() {
	collectdir_value = Utility.getCollectionDir(Gatherer.config.gsdl_path);
	// If we have a collection open gets its details.
	if(Gatherer.c_man.ready()) {
	    archivedir_value = Gatherer.c_man.getCollectionArchive();
	    builddir_value = Gatherer.c_man.getCollectionBuild();
	    importdir_value = Gatherer.c_man.getCollectionImport();
	}
    }
    /** This method returns a copy of this current <strong>BuildOptions</strong> object. Tried using stupid <i>Clonable</i> interface but it didn't work.
	  * @return A <strong>BuildOptions</strong> object with the same settings as this one.
	  */
    public BuildOptions copy() {
	BuildOptions options = new BuildOptions();
	options.allclassifications = allclassifications;
	options.archivedir = archivedir;
	options.builddir = builddir;
	options.debug = debug;
	options.collectdir = collectdir;
	options.create_images = create_images;
	options.groupsize = groupsize;
	options.gzip = gzip;
	options.importdir = importdir;
	options.index = index;
	options.keepold = keepold;
	options.maxdocs = maxdocs;
	options.mode = mode;
	options.notext = notext;
	options.oidtype = oidtype;
	options.out = out;
	options.removeold = removeold;
	options.sortmeta = sortmeta;
	options.verbosity = verbosity;
	options.groupsize_value = groupsize_value;
	options.maxdocs_value = maxdocs_value;
	options.verbosity_value = verbosity_value;
	options.archivedir_value = archivedir_value;
	options.builddir_value = builddir_value;
	options.collectdir_value = collectdir_value;
	options.importdir_value = importdir_value;
	options.mode_value = mode_value;
	options.oidtype_value = oidtype_value;
	options.out_value = out_value;
	options.sortmeta_value = sortmeta_value;
	options.index_value = index_value;
	return options;
    }
    /** When called this method returns the required arguments selected by the user. Only those arguments that are selected are returned, and even then some are never returned (i.e -out which is handled within the GShells instead).
	  * @param type The type of process that will recieve these arguments as a <strong>byte</strong>.
	  * @return A <strong>String[]</strong> containing the arguments and any required parameters as set by the user.
	  */
    public String[] getArguments(byte type) {
	ArrayList arguments = new ArrayList();
	if(allclassifications && type == BUILD_ARGS) {
	    arguments.add("-allclassifications");
	}
	if(archivedir) {
	    arguments.add("-archivedir");
	    arguments.add(archivedir_value);
	}
	if(builddir && type == BUILD_ARGS) {
	    arguments.add("-builddir");
	    arguments.add(builddir_value);
	}
	if(debug) {
	    arguments.add("-debug");
	}
	if(collectdir) {
	    arguments.add("-collectdir");
	    arguments.add(collectdir_value);
	}
	if(create_images && type == BUILD_ARGS) {
	    arguments.add("-create_images");
	} 
	if(groupsize && type == IMPORT_ARGS) {
	    arguments.add("-groupsize");
	    arguments.add(String.valueOf(groupsize_value));
	}
	if(gzip && type == IMPORT_ARGS) {
	    arguments.add("-gzip");
	}
	// Always ignore importdir eh?
	if(false && importdir && type == IMPORT_ARGS) {
	    arguments.add("-importdir");
	    arguments.add(importdir_value);
	}
	if(index && type == BUILD_ARGS) {
	    arguments.add("-index");
	    StringBuffer temp = new StringBuffer("");
	    for(int i = 0; i < index_value.length; i++) {
		temp.append(index_value[i]);
		temp.append(",");
	    }
				// Remember to remove extra ','
	    arguments.add(temp.substring(0, temp.length() - 1));
	}
	if(keepold) {
	    arguments.add("-keepold");
	}
	if(maxdocs) {
	    arguments.add("-maxdocs");
	    arguments.add(String.valueOf(maxdocs_value));
	}
	if(mode && type == BUILD_ARGS) {
	    arguments.add("-mode");
	    arguments.add(mode_value);
	}
	if(notext && type == BUILD_ARGS) {
	    arguments.add("-no_text");
	}
	if(oidtype && type == IMPORT_ARGS) {
	    arguments.add("-OIDtype");
	    arguments.add(oidtype_value);
	}
	// Never do out as its handled by the GShell.
	if(removeold && type == IMPORT_ARGS) {
	    arguments.add("-removeold");
	}
	if(sortmeta && type == IMPORT_ARGS) {
	    arguments.add("-sortmeta");
	    arguments.add(sortmeta_value);
	}
	if(verbosity) {
	    arguments.add("-verbosity");
	    arguments.add(String.valueOf(verbosity_value));
	}
	return ArrayTools.arrayListToStringArray(arguments);
    }
}
