package org.greenstone.gatherer.gui.messages;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import java.awt.*;
import java.awt.event.*;
import java.util.Date;
import javax.swing.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.Message;
import org.greenstone.gatherer.util.Utility;
/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 1.3
 */
public class MessagePane
    extends JPanel
    implements ActionListener {

    GridBagConstraints grid_constraints = null;

    GridBagLayout grid_layout = null;

    JButton save_messages = null;

    JLabel level_label = null;
    JLabel message_count = null;
    JLabel source_label = null;

    JPanel central_pane = null;
    JPanel corner_pane = null;
    JPanel count_pane = null;
    JPanel horizontal_pane = null;
    JPanel level_pane = null;
    JPanel source_pane = null;
    JPanel vertical_pane = null;

    JScrollPane scroll_pane = null;

    JToggleButton level_event = null;
    JToggleButton level_error = null;
    JToggleButton level_information = null;
    JToggleButton level_main = null;
    JToggleButton source_browser = null;
    JToggleButton source_build = null;
    JToggleButton source_general = null;
    JToggleButton source_mirroring = null;
    JToggleButton source_metaedit = null;
    JToggleButton source_session = null;

    JList message_list = null;

    MessageListModel model = null;

    MessageListRenderer renderer = null;

    String args[];

    public MessagePane() {
    }

    public void display() {
	// Setup main layout managers
	grid_layout = new GridBagLayout();
	grid_constraints = new GridBagConstraints();
	this.setLayout(grid_layout);

	// Vertical Buttons
	source_pane = new JPanel(new BorderLayout());
	source_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,3));

	vertical_pane = new JPanel(new GridLayout(6,1));

	Gatherer.println("\tCreating source_label");
	source_label = new JLabel(get("Source"));
	source_label.setHorizontalAlignment(JLabel.CENTER);

	Gatherer.println("\tCreating source_general");
	source_general = new JToggleButton(get("General"), Gatherer.config.get("messages.source_general", false));
	source_general.setIcon(Utility.OFF_ICON);
	source_general.setSelectedIcon(Utility.ON_ICON);
	source_general.setHorizontalTextPosition(JToggleButton.CENTER);
	source_general.setVerticalTextPosition(JToggleButton.BOTTOM);
	source_general.addActionListener(this);

	Gatherer.println("\tCreating source_browser");
	source_browser = new JToggleButton(get("Browser"), Gatherer.config.get("messages.source_browser", false));
	source_browser.setIcon(Utility.OFF_ICON);
	source_browser.setSelectedIcon(Utility.ON_ICON);
	source_browser.setHorizontalTextPosition(JToggleButton.CENTER);
	source_browser.setVerticalTextPosition(JToggleButton.BOTTOM);
	source_browser.addActionListener(this);

	Gatherer.println("\tCreating source_mirroring");
	source_mirroring = new JToggleButton(get("Mirroring"), Gatherer.config.get("messages.source_mirroring", false));
	source_mirroring.setIcon(Utility.OFF_ICON);
	source_mirroring.setSelectedIcon(Utility.ON_ICON);
	source_mirroring.setHorizontalTextPosition(JToggleButton.CENTER);
	source_mirroring.setVerticalTextPosition(JToggleButton.BOTTOM);
	source_mirroring.addActionListener(this);

	Gatherer.println("\tCreating source_session");
	source_session = new JToggleButton(get("Session"), Gatherer.config.get("messages.source_session", false));
	source_session.setIcon(Utility.OFF_ICON);
	source_session.setSelectedIcon(Utility.ON_ICON);
	source_session.setHorizontalTextPosition(JToggleButton.CENTER);
	source_session.setVerticalTextPosition(JToggleButton.BOTTOM);
	source_session.addActionListener(this);

	Gatherer.println("\tCreating source_metaedit");
	source_metaedit = new JToggleButton(get("Metaedit"), Gatherer.config.get("messages.source_metaedit", false));
	source_metaedit.setIcon(Utility.OFF_ICON);
	source_metaedit.setSelectedIcon(Utility.ON_ICON);
	source_metaedit.setHorizontalTextPosition(JToggleButton.CENTER);
	source_metaedit.setVerticalTextPosition(JToggleButton.BOTTOM);
	source_metaedit.addActionListener(this);

	Gatherer.println("\tCreating source_build");
	source_build = new JToggleButton(get("Build"), Gatherer.config.get("messages.source_build", false));
	source_build.setIcon(Utility.OFF_ICON);
	source_build.setSelectedIcon(Utility.ON_ICON);
	source_build.setHorizontalTextPosition(JToggleButton.CENTER);
	source_build.setVerticalTextPosition(JToggleButton.BOTTOM);
	source_build.addActionListener(this);

	source_pane.add(source_label, BorderLayout.NORTH);
	vertical_pane.add(source_general);
	vertical_pane.add(source_browser);
	vertical_pane.add(source_mirroring);
	vertical_pane.add(source_session);
	vertical_pane.add(source_metaedit);
	vertical_pane.add(source_build);
	source_pane.add(vertical_pane, BorderLayout.CENTER);

	// Central pane
	central_pane = new JPanel(new BorderLayout());

	Gatherer.println("\tCreating message_list");
	model = new MessageListModel();
	renderer = new MessageListRenderer();
	message_list = new JList();
	message_list.setModel(model);
	message_list.setCellRenderer(renderer);

	scroll_pane = new JScrollPane(message_list);
	central_pane.add(scroll_pane, BorderLayout.CENTER);

	Gatherer.println("\tCreating message_count");
	count_pane = new JPanel(new BorderLayout());
	args = new String[2];
	args[0] = model.getSizeStr();
	args[1] = model.getTotalSizeStr();
	message_count = new JLabel(get("Message_Count",args));
	message_count.setHorizontalAlignment(JLabel.RIGHT);

	central_pane.add(scroll_pane, BorderLayout.CENTER);
	central_pane.add(message_count, BorderLayout.SOUTH);

	// Corner area
	corner_pane = new JPanel(new BorderLayout());
	corner_pane.setBorder(BorderFactory.createEmptyBorder(3,5,5,5));

	Gatherer.println("\tCreating save_message");
	save_messages = new JButton(get("Save_Messages"),
				    new ImageIcon(Utility.RES_DIR+"save.gif"));
	save_messages.addActionListener(this);

	corner_pane.add(save_messages, BorderLayout.CENTER);

	// Horizontal buttons.
	level_pane = new JPanel(new BorderLayout());

	horizontal_pane = new JPanel(new GridLayout(1,4));

	Gatherer.println("\tCreating level_label");
	level_label = new JLabel(get("Level"));
	level_label.setHorizontalAlignment(JLabel.CENTER);

	Gatherer.println("\tCreating level_main");
	level_main = new JToggleButton(get("Main"), Gatherer.config.get("messages.level_main", false));
	level_main.setIcon(Utility.OFF_ICON);
	level_main.setSelectedIcon(Utility.ON_ICON);
	level_main.addActionListener(this);

	Gatherer.println("\tCreating level_event");
	level_event = new JToggleButton(get("Event"), Gatherer.config.get("messages.level_event", false));
	level_event.setIcon(Utility.OFF_ICON);
	level_event.setSelectedIcon(Utility.ON_ICON);
	level_event.addActionListener(this);

	Gatherer.println("\tCreating level_information");
	level_information = new JToggleButton(get("Information"), Gatherer.config.get("messages.level_information", false));
	level_information.setIcon(Utility.OFF_ICON);
	level_information.setSelectedIcon(Utility.ON_ICON);
	level_information.addActionListener(this);

	Gatherer.println("\tCreating level_error");
	level_error = new JToggleButton(get("Error"), Gatherer.config.get("messages.level_error", false));
	level_error.setIcon(Utility.OFF_ICON);
	level_error.setSelectedIcon(Utility.ON_ICON);
	level_error.addActionListener(this);

	level_pane.add(level_label, BorderLayout.NORTH);
	horizontal_pane.add(level_main);
	horizontal_pane.add(level_event);
	horizontal_pane.add(level_information);
	horizontal_pane.add(level_error);
	level_pane.add(horizontal_pane, BorderLayout.CENTER);

	// Terrifying gridbag layout
	grid_constraints.fill = GridBagConstraints.BOTH;
	grid_layout.setConstraints(source_pane, grid_constraints);
	this.add(source_pane);
	grid_constraints.weightx = 1.0;
	grid_constraints.weighty = 1.0;
	grid_constraints.gridwidth = GridBagConstraints.REMAINDER;
	grid_layout.setConstraints(central_pane, grid_constraints);
	this.add(central_pane);
	grid_constraints.weightx = 0.0;
	grid_constraints.weighty = 0.0;
	grid_constraints.gridwidth = 1;
	grid_layout.setConstraints(corner_pane, grid_constraints);
	this.add(corner_pane);
	grid_constraints.gridwidth = GridBagConstraints.REMAINDER;
	grid_layout.setConstraints(level_pane, grid_constraints);
	this.add(level_pane);
    }

    public void actionPerformed(ActionEvent event) {
	Object source = event.getSource();

	// Eventually this will dump the messages to a text file, but for now
	// I'm using it in testing.
	if(source == save_messages) {
	    for(int i = 0; i < 20; i++) {
		model.add(Message.random());
		message_list.ensureIndexIsVisible(model.getSize() - 1);
	    }
	}
	// We've been asked to toggle whether we show general messages.
	else if(source == source_general) {
				// Toggle config setting
	    Gatherer.config.set("messages.source_general", false, !Gatherer.config.get("messages.source_general", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// We've been asked to toggle whether we show browser messages.
	else if(source == source_browser) {
				// Toggle config setting
	    Gatherer.config.set("messages.source_browser", false, !Gatherer.config.get("messages.source_browser", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// We've been asked to toggle whether we show general messages.
	else if(source == source_mirroring) {
				// Toggle config setting
	    Gatherer.config.set("messages.source_mirroring", false, !Gatherer.config.get("messages.source_mirroring", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// We've been asked to toggle whether we show session messages.
	else if(source == source_session) {
				// Toggle config setting
	    Gatherer.config.set("messages.source_session", false, !Gatherer.config.get("messages.source_session", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// We've been asked to toggle whether we show metaedit messages.
	else if(source == source_metaedit) {
				// Toggle config setting
	    Gatherer.config.set("messages.source_metaedit", false, !Gatherer.config.get("messages.source_metaedit", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// We've been asked to toggle whether we show build messages.
	else if(source == source_build) {
				// Toggle config setting
	    Gatherer.config.set("messages.source_build", false, !Gatherer.config.get("messages.source_build", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// We've been asked to toggle whether we show main level messages.
	else if(source == level_main) {
				// Toggle config setting
	    Gatherer.config.set("messages.level_main", false, !Gatherer.config.get("messages.level_main", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// We've been asked to toggle whether we show event level messages.
	else if(source == level_event) {
				// Toggle config setting
	    Gatherer.config.set("messages.level_event", false, !Gatherer.config.get("messages.level_event", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// We've been asked to toggle whether we show information level messages.
	else if(source == level_information) {
				// Toggle config setting
	    Gatherer.config.set("messages.level_information", false, !Gatherer.config.get("messages.level_information", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// We've been asked to toggle whether we show error level messages.
	else if(source == level_error) {
				// Toggle config setting
	    Gatherer.config.set("messages.level_error", false, !Gatherer.config.get("messages.level_error", false));
				// Inform model something has changed.
	    model.refresh();
	    message_list.ensureIndexIsVisible(model.getSize() - 1);
	}
	// Pretty much no matter what happens we need to get the new size
	args = new String[2];
	args[0] = model.getSizeStr();
	args[1] = model.getTotalSizeStr();
	message_count.setText(get("Message_Count",args));
    }

    /** The callback for the message listener. Here all of the messages are
     * displayed onto the list.
     * @param message The Message to be displayed.
     */
    public void show(Message message) {
	model.add(message);
	message_list.ensureIndexIsVisible(model.getSize() - 1);
    }


    private String get(String key) {
	return get(key, null);
    }

    private String get(String key, String args[]) {
	if(key.indexOf('.') == -1) {
	    key = "Messages." + key;
	}
	return Gatherer.dictionary.get(key, args);
    }
}

